#!/usr/bin/env python

import dbus.glib
import gobject
import dbus
import dbus.service
import gtk
import sys
import signal

class TransferDialog(gtk.Dialog):
    def __init__(self, bus, object_path='/com/nokia/ObexHandler'):
        gtk.Dialog.__init__("Object Transfer")

        self.bus = bus
        self.xfers = []

        self.connect('destroy', lambda self: gtk.main_quit())

    def __get_xfer(self, path):
        return path in self.xfers

    def transfer_started(self, success):
        if success:
            print'TransferStarted: success'
        else:
            print'TransferStarted: failure'

    def transfer_complete(self, success):
        if success:
            print'TransferComplete: success'
        else:
            print'TransferComplete: failure'

    def transfer_progress(self, current, final):
        print 'TransferProgress: %d/%d' % (current, final)

    def new_transfer(self, path, addr, name, type, length, time):
        xfer = self.bus.get_object('com.nokia.ObexServer', path)
        xfer.connect_to_signal('TransferStarted', self.transfer_started)
        xfer.connect_to_signal('TransferComplete', self.transfer_complete)
        xfer.connect_to_signal('TransferProgress', self.transfer.progress)
        self.xfers.append(xfer)

class ServerTest(dbus.service.Object):
    def __init__(self):
        self.path = "/com/nokia/ObexHandler"
        self.bus = dbus.SystemBus()

        for name in self.bus._bus_names:
            print name

        un = self.bus.get_connection().get_unique_name()
        if not un in self.bus._bus_names:
            print "Unique name not registered!?"
            sys.exit(1)

        bus_name = self.bus._bus_names[un]

        dbus.service.Object.__init__(self, bus_name, self.path)

        self.dialog = TransferDialog(self.bus)

        obj = self.bus.get_object('com.nokia.ObexServer', '/com/nokia/ObexServer')
        self.srv = dbus.Interface(self.obj, 'com.nokia.ObexServer')

        self.srv.RegisterHandler(self.path)

    def exit(self, signal):
        self.srv.UnregisterHandler(self.path)
        gtk.main.quit()

    @dbus.service.method('com.nokia.ObexServer.Handler')
    def Authorize(self, path, addr, name, type, length, time):
        self.xfers.append(path)

        self.dialog.new_transfer(path, addr, name, type, length, time)

        self.dialog.run()

        return self.dialog.get_filename(path)

if __name__ == '__main__':
    gobject.threads_init()
    dbus.glib.init_threads()
    app = ServerTest()
    signal.signal(signal.SIGINT, app.exit)
    gtk.main()
